# -*- coding: utf-8 -*-
# @Author: wangkun
# @Time: 2022/3/29
"""
从 微信小程序-快手短视频 中，下载符合规则的视频
"""
import json
import os
import sys
import time
import requests
import urllib3
sys.path.append(os.getcwd())
from main.common import Common
from main.feishu_lib import Feishu
from main.publish import Publish

proxies = {"http": None, "https": None}


class KuaiShou:
    # 已下载视频列表
    download_video_list = []
    # 配置微信号
    Referer = Feishu.get_range_value("f1R7Mx", "C3:C3")[0][0]["link"]
    NS_sig3 = Feishu.get_range_value("f1R7Mx", "C4:C4")[0]
    NS_sig3_origin = Feishu.get_range_value("f1R7Mx", "C5:C5")[0]
    did = Feishu.get_range_value("f1R7Mx", "C6:C6")[0]
    session_key = Feishu.get_range_value("f1R7Mx", "C7:C7")[0]
    unionid = Feishu.get_range_value("f1R7Mx", "C8:C8")[0]
    eUserStableOpenId = Feishu.get_range_value("f1R7Mx", "C9:C9")[0]
    openId = Feishu.get_range_value("f1R7Mx", "C10:C10")[0]
    eOpenUserId = Feishu.get_range_value("f1R7Mx", "C11:C11")[0]
    kuaishou_wechat_app_st = Feishu.get_range_value("f1R7Mx", "C12:C12")[0]
    passToken = Feishu.get_range_value("f1R7Mx", "C13:C13")[0]
    userId = Feishu.get_range_value("f1R7Mx", "C14:C14")[0]

    @classmethod
    def sensitive_words(cls):
        # 敏感词库列表
        word_list = []
        # 从云文档读取所有敏感词，添加到词库列表
        lists = Feishu.get_values_batch("fn8IDi")
        for i in lists:
            for j in i:
                # 过滤空的单元格内容
                if j is None:
                    pass
                else:
                    word_list.append(j)
        return word_list

    @staticmethod
    def kuaishou_download_rule(d_duration, d_width, d_height,
                               d_play_cnt, d_like_cnt, d_share_cnt):
        """
        下载视频的基本规则
        :param d_duration: 时长
        :param d_width: 宽
        :param d_height: 高
        :param d_play_cnt: 播放量
        :param d_like_cnt: 点赞量
        :param d_share_cnt: 分享量
        :return: 满足规则，返回 True；反之，返回 False
        """
        if 600 >= int(float(d_duration)) >= 60:
            if int(d_width) >= 720 or int(d_height) >= 720:
                if int(d_play_cnt) >= 50000:
                    if int(d_like_cnt) >= 50000:
                        if int(d_share_cnt) >= 2000:
                            return True
                        else:
                            return False
                    else:
                        return False
                else:
                    return False
            return False
        return False

    @classmethod
    def get_feeds(cls):
        """
        1.从快手小程序首页推荐，获取视频列表
        2.先在 https://w42nne6hzg.feishu.cn/sheets/shtcnp4SaJt37q6OOOrYzPMjQkg?sheet=3b207c 中去重
        3.再从 https://w42nne6hzg.feishu.cn/sheets/shtcnp4SaJt37q6OOOrYzPMjQkg?sheet=Zt2PGQ 中去重
        4.添加视频信息至 https://w42nne6hzg.feishu.cn/sheets/shtcnp4SaJt37q6OOOrYzPMjQkg?sheet=Zt2PGQ
        """
        url = "https://wxmini-api.uyouqu.com/rest/wd/wechatApp/feed/recommend"
        headers = {
            "content-type": "application/json",
            "Accept-Encoding": "gzip,compress,br,deflate",
            "User-Agent": 'Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X)'
                          ' AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148'
                          ' MicroMessenger/8.0.20(0x18001442) NetType/WIFI Language/zh_CN',
            "Referer": str(cls.Referer),
        }
        params = {
            "__NS_sig3": str(cls.NS_sig3),
            "__NS_sig3_origin": str(cls.NS_sig3_origin)
        }
        cookies = {
            "did": str(cls.did),
            "preMinaVersion": "v3.109.0",
            "sid": "kuaishou.wechat.app",
            "appId": "ks_wechat_small_app_2",
            "clientid": "13",
            "client_key": "f60ac815",
            "kpn": "WECHAT_SMALL_APP",
            "kpf": "OUTSIDE_ANDROID_H5",
            "language": "zh_CN",
            "smallAppVersion": "v3.114.0",
            "session_key": str(cls.session_key),
            "unionid": str(cls.unionid),
            "eUserStableOpenId": str(cls.eUserStableOpenId),
            "openId": str(cls.openId),
            "eOpenUserId": str(cls.eOpenUserId),
            "kuaishou.wechat.app_st": str(cls.kuaishou_wechat_app_st),
            "passToken": str(cls.passToken),
            "userId": str(cls.userId)
        }
        json_data = {
            "count": 10,
            "portal": 1,
            "pageType": 2,
            "extraRequestInfo": "{\"scene\":1089,\"fid\":\"\",\"sharerUserId\":\"\",\"curPhotoIndex\":0,"
                                "\"adShow\":true,\"weChatAd\":{},\"headurl\":\"https://js2.a.kwimgs.com/udata/pkg"
                                "/fe/profiel_icon_photo_normal@3x.fb3ec1af.png\",\"page\":0}",
            "needLivestream": True,
            "pcursor": 0,
            "sourceFrom": 2,
            "thirdPartyUserId": int(cls.userId)
        }

        try:
            urllib3.disable_warnings()
            r = requests.post(url=url, headers=headers, params=params,
                              cookies=cookies, json=json_data, proxies=proxies, verify=False)
            response = json.loads(r.content.decode("utf8"))
            feeds = response["feeds"]
            for i in range(len(feeds)):
                # 视频标题过滤话题及处理特殊字符
                kuaishou_title = feeds[i]["caption"]
                title_split1 = kuaishou_title.split(" #")
                if title_split1[0] != "":
                    title1 = title_split1[0]
                else:
                    title1 = title_split1[-1]

                title_split2 = title1.split(" #")
                if title_split2[0] != "":
                    title2 = title_split2[0]
                else:
                    title2 = title_split2[-1]

                title_split3 = title2.split("@")
                if title_split3[0] != "":
                    title3 = title_split3[0]
                else:
                    title3 = title_split3[-1]

                video_title = title3.strip().replace("\n", "") \
                    .replace("/", "").replace("快手", "").replace(" ", "") \
                    .replace(" ", "").replace("&NBSP", "").replace("\r", "") \
                    .replace("#", "").replace(".", "。").replace("\\", "") \
                    .replace(":", "").replace("*", "").replace("？", "") \
                    .replace("?", "").replace('"', "").replace("<", "") \
                    .replace(">", "").replace("|", "")
                Common.logger().info("video_title:{}".format(video_title))

                if "photoId" not in feeds[i]:
                    photo_id = "0"
                    Common.logger().info("photo_id:{}".format(photo_id))
                else:
                    photo_id = feeds[i]["photoId"]
                    Common.logger().info("photo_id:{}".format(photo_id))

                if "viewCount" not in feeds[i]:
                    video_play_cnt = "0"
                    Common.logger().info("video_play_cnt:0")
                else:
                    video_play_cnt = feeds[i]["viewCount"]
                    Common.logger().info("video_play_cnt:{}".format(video_play_cnt))

                if "likeCount" not in feeds[i]:
                    video_like_cnt = "0"
                    Common.logger().info("video_like_cnt:0")
                else:
                    video_like_cnt = feeds[i]["likeCount"]
                    Common.logger().info("video_like_cnt:{}".format(video_like_cnt))

                if "shareCount" not in feeds[i]:
                    video_share_cnt = "0"
                    Common.logger().info("video_share_cnt:0")
                else:
                    video_share_cnt = feeds[i]["shareCount"]
                    Common.logger().info("video_share_cnt:{}".format(video_share_cnt))

                if "commentCount" not in feeds[i]:
                    video_comment_cnt = "0"
                    Common.logger().info("video_comment_cnt:0")
                else:
                    video_comment_cnt = feeds[i]["commentCount"]
                    Common.logger().info("video_comment_cnt:{}".format(video_comment_cnt))

                if "duration" not in feeds[i]:
                    video_duration = "0"
                    Common.logger().info("video_duration:不存在")
                else:
                    video_duration = int(int(feeds[i]["duration"]) / 1000)
                    Common.logger().info("video_duration:{}秒".format(video_duration))

                if "width" not in feeds[i] or "height" not in feeds[i]:
                    video_width = "0"
                    video_height = "0"
                    video_resolution = str(video_width) + "*" + str(video_height)
                    Common.logger().info("无分辨率")
                else:
                    video_width = feeds[i]["width"]
                    video_height = feeds[i]["height"]
                    video_resolution = str(video_width) + "*" + str(video_height)
                    Common.logger().info("video_resolution:{}".format(video_resolution))

                if "timestamp" not in feeds[i]:
                    video_send_time = "0"
                    Common.logger().info("video_send_time:不存在")
                else:
                    video_send_time = feeds[i]["timestamp"]
                    Common.logger().info("video_send_time:{}".format(
                        time.strftime("%Y/%m/%d %H:%M:%S", time.localtime(int(video_send_time) / 1000))))

                user_name = feeds[i]["userName"].strip().replace("\n", "") \
                    .replace("/", "").replace("快手", "").replace(" ", "") \
                    .replace(" ", "").replace("&NBSP", "").replace("\r", "")
                Common.logger().info("user_name:{}".format(user_name))

                user_id = feeds[i]["userId"]
                Common.logger().info("user_id:{}".format(user_id))

                if "headUrl" not in feeds[i]:
                    head_url = "0"
                    Common.logger().info("head_url:不存在")
                else:
                    head_url = feeds[i]["headUrl"]
                    Common.logger().info("head_url:{}".format(head_url))

                if len(feeds[i]["coverUrls"]) == 0:
                    cover_url = "0"
                    Common.logger().info("cover_url:不存在")
                else:
                    cover_url = feeds[i]["coverUrls"][0]["url"]
                    Common.logger().info("cover_url:{}".format(cover_url))

                if len(feeds[i]["mainMvUrls"]) == 0:
                    video_url = "0"
                    Common.logger().info("video_url:不存在")
                else:
                    video_url = feeds[i]["mainMvUrls"][0]["url"]
                    Common.logger().info("video_url:{}".format(video_url))

                # 过滤无效视频
                if photo_id == "0" \
                        or head_url == "0" \
                        or cover_url == "0" \
                        or video_url == "0" \
                        or video_duration == "0" \
                        or video_send_time == "0" \
                        or user_name == "" \
                        or video_title == "":
                    Common.logger().info("无效视频")
                # 判断敏感词
                elif any(word if word in kuaishou_title else False for word in cls.sensitive_words()) is True:
                    Common.logger().info("视频已中敏感词：{}".format(kuaishou_title))
                # 从 云文档 去重:https://w42nne6hzg.feishu.cn/sheets/shtcnp4SaJt37q6OOOrYzPMjQkg?sheet=3b207c
                elif photo_id in [j for i in Feishu.get_values_batch("3b207c") for j in i]:
                    Common.logger().info("该视频已下载：{}", video_title)
                # 从 云文档 去重:https://w42nne6hzg.feishu.cn/sheets/shtcnp4SaJt37q6OOOrYzPMjQkg?sheet=Zt2PGQ
                elif photo_id in [j for i in Feishu.get_values_batch("Zt2PGQ") for j in i]:
                    Common.logger().info("该视频已在feeds中：{}", video_title)
                else:
                    Common.logger().info("该视频未下载，添加至feeds中:{}".format(video_title))
                    # feeds工作表，插入首行
                    time.sleep(1)
                    Feishu.insert_columns("Zt2PGQ", "ROWS", 1, 2)

                    # 获取当前时间
                    get_feeds_time = int(time.time())
                    # 看一看云文档，工作表 kanyikan_feeds_1 中写入数据
                    values = [[str(time.strftime("%Y/%m/%d %H:%M:%S", time.localtime(get_feeds_time))),
                               "推荐榜",
                               photo_id,
                               video_title,
                               video_play_cnt,
                               video_comment_cnt,
                               video_like_cnt,
                               video_share_cnt,
                               video_duration,
                               video_resolution,
                               time.strftime("%Y/%m/%d %H:%M:%S", time.localtime(int(video_send_time) / 1000)),
                               user_name,
                               user_id,
                               head_url,
                               cover_url,
                               video_url]]
                    # 等待 1s，防止操作云文档太频繁，导致报错
                    time.sleep(1)
                    Feishu.update_values("Zt2PGQ", "A2:P2", values)
        except Exception as e:
            Common.logger().error("获取视频 list 异常:{}".format(e))

    @classmethod
    def download_publish(cls, env):
        """
        1.从 https://w42nne6hzg.feishu.cn/sheets/shtcnp4SaJt37q6OOOrYzPMjQkg?sheet=Zt2PGQ 中读取视频信息
        2.下载并上传符合规则的视频
        测试环境:env == dev
        正式环境:env == prod
        """
        try:
            if len(Feishu.get_values_batch("Zt2PGQ")) == 1:
                pass
            else:
                for i in range(1, len(Feishu.get_values_batch("Zt2PGQ")) + 1):
                    time.sleep(1)
                    try:
                        download_photo_id = Feishu.get_values_batch("Zt2PGQ")[i][2]
                        download_video_title = Feishu.get_values_batch("Zt2PGQ")[i][3]
                        download_video_play_cnt = Feishu.get_values_batch("Zt2PGQ")[i][4]
                        download_video_comment_cnt = Feishu.get_values_batch("Zt2PGQ")[i][5]
                        download_video_like_cnt = Feishu.get_values_batch("Zt2PGQ")[i][6]
                        download_video_share_cnt = Feishu.get_values_batch("Zt2PGQ")[i][7]
                        download_video_duration = Feishu.get_values_batch("Zt2PGQ")[i][8]
                        download_video_resolution = Feishu.get_values_batch("Zt2PGQ")[i][9]
                        download_video_width = download_video_resolution.split("*")[0]
                        download_video_height = download_video_resolution.split("*")[-1]
                        download_video_send_time = Feishu.get_values_batch("Zt2PGQ")[i][10]
                        download_user_name = Feishu.get_values_batch("Zt2PGQ")[i][11]
                        download_user_id = Feishu.get_values_batch("Zt2PGQ")[i][12]
                        download_head_url = Feishu.get_values_batch("Zt2PGQ")[i][13][0]["link"]
                        download_cover_url = Feishu.get_values_batch("Zt2PGQ")[i][14][0]["link"]
                        download_video_url = Feishu.get_values_batch("Zt2PGQ")[i][15][0]["link"]

                        # Common.logger().info("download_photo_id:{}", download_photo_id)
                        # Common.logger().info("download_video_title:{}", download_video_title)
                        # Common.logger().info("download_video_play_cnt:{}", download_video_play_cnt)
                        # Common.logger().info("download_video_comment_cnt:{}", download_video_comment_cnt)
                        # Common.logger().info("download_video_like_cnt:{}", download_video_like_cnt)
                        # Common.logger().info("download_video_share_cnt:{}", download_video_share_cnt)
                        # Common.logger().info("download_video_duration:{}", download_video_duration)
                        # Common.logger().info("download_video_resolution:{}", download_video_resolution)
                        # Common.logger().info("download_video_send_time:{}", download_video_send_time)
                        # Common.logger().info("download_user_name:{}", download_user_name)
                        # Common.logger().info("download_user_id:{}", download_user_id)
                        # Common.logger().info("download_head_url:{}", download_head_url)
                        # Common.logger().info("download_cover_url:{}", download_cover_url)
                        # Common.logger().info("download_video_url:{}", download_video_url)

                        Common.logger().info("正在判断第{}行，视频:{}", i, download_video_title)

                        # 过滤空行
                        if download_photo_id is None or download_video_title is None or download_video_play_cnt is None:
                            Common.logger().warning("空行，略过")
                        # 去重
                        elif download_photo_id in [j for i in Feishu.get_values_batch("3b207c") for j in i]:
                            Common.logger().info("该视频已下载:{}", download_video_title)
                        # 下载规则
                        elif cls.kuaishou_download_rule(
                                download_video_duration, download_video_width, download_video_height,
                                download_video_play_cnt, download_video_like_cnt, download_video_share_cnt) is True:
                            Common.logger().info("开始下载快手视频:{}".format(download_video_title))

                            # 下载封面
                            Common.download_method(text="cover",
                                                   d_name=str(download_video_title), d_url=str(download_cover_url))
                            # 下载视频
                            Common.download_method(text="video",
                                                   d_name=str(download_video_title), d_url=str(download_video_url))
                            # 保存视频信息至 "./videos/{download_video_title}/info.txt"
                            with open("./videos/" + download_video_title
                                      + "/" + "info.txt", "a", encoding="UTF-8") as f_a:
                                f_a.write(str(download_photo_id) + "\n" +
                                          str(download_video_title) + "\n" +
                                          str(download_video_duration) + "\n" +
                                          str(download_video_play_cnt) + "\n" +
                                          str(download_video_comment_cnt) + "\n" +
                                          str(download_video_like_cnt) + "\n" +
                                          str(download_video_share_cnt) + "\n" +
                                          str(download_video_resolution) + "\n" +
                                          str(int(time.mktime(
                                              time.strptime(download_video_send_time, "%Y/%m/%d %H:%M:%S")))) + "\n" +
                                          str(download_user_name) + "\n" +
                                          str(download_head_url) + "\n" +
                                          str(download_video_url) + "\n" +
                                          str(download_cover_url) + "\n" +
                                          str(cls.did))
                            Common.logger().info("==========视频信息已保存至info.txt==========")

                            # 添加视频 ID 到 list，用于统计当次下载总数
                            cls.download_video_list.append(download_photo_id)

                            # 上传视频
                            Common.logger().info("开始上传视频:{}".format(download_video_title))
                            Publish.upload_and_publish(env, "play")

                            # 保存视频 ID 到云文档:https://w42nne6hzg.feishu.cn/sheets/shtcnp4SaJt37q6OOOrYzPMjQkg?sheet=3b207c
                            Common.logger().info("保存视频ID至云文档:{}", download_video_title)
                            # 视频ID工作表，插入首行
                            Feishu.insert_columns("3b207c", "ROWS", 1, 2)
                            # 视频ID工作表，首行写入数据
                            upload_time = int(time.time())
                            values = [[str(time.strftime("%Y/%m/%d %H:%M:%S", time.localtime(upload_time))),
                                       "推荐榜",
                                       str(download_photo_id),
                                       str(download_video_title),
                                       str(download_video_play_cnt),
                                       str(download_video_comment_cnt),
                                       str(download_video_like_cnt),
                                       str(download_video_share_cnt),
                                       str(download_video_duration),
                                       str(download_video_resolution),
                                       str(download_video_send_time),
                                       str(download_user_name),
                                       str(download_user_id),
                                       str(download_head_url),
                                       str(download_cover_url),
                                       str(download_video_url)]]
                            time.sleep(1)
                            Feishu.update_values("3b207c", "A2:Q2", values)

                            # 从云文档删除该视频信息:https://w42nne6hzg.feishu.cn/sheets/shtcngRPoDYAi24x52j2nDuHMih?sheet=Zt2PGQ
                            Common.logger().info("从云文档删除该视频信息:{}", download_video_title)
                            # 删除行或列，可选 ROWS、COLUMNS
                            time.sleep(1)
                            Feishu.dimension_range("Zt2PGQ", "ROWS", i + 1, i + 1)
                        else:
                            # 从云文档删除该视频信息:https://w42nne6hzg.feishu.cn/sheets/shtcngRPoDYAi24x52j2nDuHMih?sheet=Zt2PGQ
                            Common.logger().info("该视频不满足下载规则，删除在云文档中的信息:{}", download_video_title)
                            # 删除行或列，可选 ROWS、COLUMNS
                            Feishu.dimension_range("Zt2PGQ", "ROWS", i + 1, i + 1)
                    except Exception as e:
                        Common.logger().error("视频 info 异常，删除该视频信息", e)
                        # 删除行或列，可选 ROWS、COLUMNS
                        Feishu.dimension_range("Zt2PGQ", "ROWS", i + 1, i + 1)
                cls.download_publish("prod")

        except Exception as e:
            Common.logger().error(e)


if __name__ == "__main__":
    kuaishou = KuaiShou()
    kuaishou.get_feeds()
    kuaishou.download_publish("dev")
